/*
 *	File transfer protocol "TransIt"
 *		Version 2.00
 *	Copyright (C) 1987-1989 by ASCII Corporation.
 *
 *
 *	main.c - main routine
 */
#include "transit.h"

int gmode;
int oopt;
int topt;
int gopt;
int vopt;
int blockAck;

#ifdef DEBUG
FILE	*logptr;
#endif

static jmp_buf mainEnv;		/* environment to return to main() */

/*
 *  signal handler
 */
sigfunc_t sigHandle()
{
	if(vopt) fputs("..User abort\n\n", stderr);
#ifdef	DEBUG
	fputs("\n..User abort\007\007\n", logptr);
						/* show message */
#endif
	longjmp(mainEnv, QUIT);			/* return with ERROR code */
}

/*
 *  terminate transfer
 */
quit(msg)
char *msg;
{
	if(vopt){
		fputs("\r\n",stderr);
		fputs(msg, stderr);
		fputs("\r\n",stderr);
	}
#ifdef DEBUG
	fputs("\n..Too many errors\007\007\n", logptr);
						/* show message */
#endif
	longjmp(mainEnv, QUIT);			/* return with ERROR code */
}

/*
 *  error abort
 */
abort()
{
	if(vopt) fputs("..Terminate transfer\r\n", stderr);
#ifdef DEBUG
	fputs("\n..Terminate transfer\007\007\r\n", logptr);
						/* show message */
#endif
	longjmp(mainEnv, ABORT);		/* return with ERROR code */
}

static void usage() /* added by Kenar */
{
	fputs("TransIt: File transfer protocol by ASCII co.\n", stderr);
	fputs(
	"usage1: transit [-hv -t <timeout> -r <retry>] r [o] [g] {<path-name>}\n",
	stderr);
	fputs(
	"usage2: transit [-hv -t <timeout> -r <retry>] s [t] [g] {<path-name>}\n",
	stderr);
	fputs("options are:\n",stderr);
	fputs("  -h: this help message\n",stderr);
	fputs("  -v: verbose mode\n",stderr);
	fputs("  -t <timeout>: time out per trial in unit of second\n",stderr);
	fputs("  -r <retry>: maximum retry\n",stderr);
	fputs("version: 2.0K (TransIt Ver.2.0 mod by Kenar)\n",stderr);
	exit(0);
}

/*
 *  main
 */
main(argc, argv)
int argc;
char *argv[];
{
	char ch, *chp;
	int result;
	char buffer[LINESIZE];
	int i;
	extern char *optarg;
	extern int optind;

	gmode = 0;
	oopt = FALSE;
	topt = FALSE;
	gopt = FALSE;
	vopt = FALSE;
	MAXRETRY = 10;
	ETIMEOUT = 10;

	/* set signal handler */
	signal(SIGINT, sigHandle);		/* set signal handler */

	/* check argument */
	while((ch = getopt(argc,argv,"hvt:r:")) != EOF){ /* added by Kenar */
		switch(ch){
		case 'v':
			vopt = TRUE;
			break;
		case 't':
			ETIMEOUT = atoi(optarg);
			break;
		case 'r':
			MAXRETRY = atoi(optarg);
			break;
		case 'h':
		default:
			usage();
		}
	}
	
	/* option count is (optind -1) */
	argc -= (optind - 1);
	argv += (optind - 1);

	if(argc < 2) usage();
	++argv;
	ch = (*argv)[0];		/* command */
	if(ch != SENDCMD && ch != RECVCMD) usage();
	for (i = 1; (*argv)[i]; i++) {
		switch ((*argv)[i]) {
		case 'o':		/* overwrite option */
			oopt = TRUE;
			break;
		case 't':
			topt = TRUE;	/* textmode option */
			break;
		case 'g':
			gopt = TRUE;	/* graph option */
			break;
		default: usage();
		}
	}
	argc -= 2;
	++argv;				/* move forward pointer */
	
	rsdAttach();

	if ((result = setjmp(mainEnv)) == 0) {		/* save environment */
		result = SUCCESS;
		if (ch == SENDCMD) {		/* send files */
			sendFile(argc, argv);
		}
		else if (ch == RECVCMD) {	/* recv files */
			recvFile(argc, argv);
		}
	}
	else {
		if (result == QUIT) {
			sendBytes(1, CONTROL, TERMINATE);
		}
		result = ERROR;
	}
	
	rsdDetach();
	exit(result);
}
